<?xml version="1.0" encoding="UTF-8"?>
<xsl:package name="com.ivans.connect.format" package-version="0.1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:dl="http://www.ivansinsurance.com/connect/download" 
    xmlns:al3="http://www.ivansinsurance.com/acord/al3"
    xmlns:format="http://www.ivansinsurance.com/acord/al3/formatters"
    xmlns:map="http://www.w3.org/2005/xpath-functions/map"
    xmlns:fn="http://www.w3.org/2005/xpath-functions" exclude-result-prefixes="#all"
    expand-text="true" version="3.0">
    <xsl:expose visibility="public" component="function" names="format:* dl:*"/>
    <xsl:use-package name="com.ivans.connect.acord.al3" package-version="0.1.0"/>
    <xsl:mode name="dl:al3-name"/>
    <xsl:function name="format:al3-name">
        <xsl:param name="value"/>
        <xsl:param name="length"/>
        <xsl:variable name="formatted-value">
            <xsl:apply-templates select="$value" mode="dl:al3-name"/>
        </xsl:variable>
        <xsl:sequence select="substring($formatted-value || '          ',1, $length)"/>
    </xsl:function>
    <xsl:function name="format:al3-name-by-type" as="xs:string">
        <xsl:param name="type" as="xs:string"/>
        <xsl:param name="name-parts" as="map(*)"/>
        <xsl:variable name="name-builder" select="$al3:name-formats($type)"/>
        <xsl:sequence
            select="
            $type ||
            fold-left($name-builder, '',
            function ($acc, $part) {
            $acc || format:text($name-parts($part(1)), $part(2))
            }
            )"
        />
    </xsl:function>
    <xsl:template match=".[. instance of map(*)]" mode="dl:al3-name">
        <!-- if the name value is a map, assume it's in the expected format -->
        <xsl:sequence select="format:al3-name-by-type(.?Type, map:remove(., 'Type'))"/>
    </xsl:template>
    <xsl:template match=".[. instance of xs:string]" mode="dl:al3-name">
        <xsl:sequence select="."/>
    </xsl:template>
    <xsl:function name="format:str" as="xs:string">
        <xsl:param name="value" as="xs:string?"/>
        <xsl:param name="length" as="xs:integer"/>
        <xsl:param name="normalize-function" as="function(*)"/>
        <xsl:sequence select="$value 
            => normalize-space() 
            => $normalize-function()
            => concat('          ')
            => substring(1, $length)"/>
    </xsl:function> 
    <xsl:function name="format:text" as="xs:string">
        <xsl:param name="value" as="xs:string?"/>
        <xsl:param name="length" as="xs:integer"/>
        <xsl:sequence select="format:str($value, $length, replace(?, '[^a-zA-Z0-9/.,=+\-_()*&amp;^%$#@!]',''))"/>
    </xsl:function>    
</xsl:package>
